# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common
from . import socket_address_object


class SocketOptionsType(GeneratedsSuper):
    """The SocketOptionsType specifies any particular options used by the
    socket. If an options is supported only by specific address
    families or socket types, that's indicated in parentheses."""

    subclass = None
    superclass = None
    def __init__(self, IP_MULTICAST_IF=None, IP_MULTICAST_IF2=None, IP_MULTICAST_LOOP=None, IP_TOS=None, SO_BROADCAST=None, SO_CONDITIONAL_ACCEPT=None, SO_KEEPALIVE=None, SO_DONTROUTE=None, SO_LINGER=None, SO_DONTLINGER=None, SO_OOBINLINE=None, SO_RCVBUF=None, SO_GROUP_PRIORITY=None, SO_REUSEADDR=None, SO_DEBUG=None, SO_RCVTIMEO=None, SO_SNDBUF=None, SO_SNDTIMEO=None, SO_UPDATE_ACCEPT_CONTEXT=None, SO_TIMEOUT=None, TCP_NODELAY=None):
        self.IP_MULTICAST_IF = IP_MULTICAST_IF
        self.IP_MULTICAST_IF2 = IP_MULTICAST_IF2
        self.IP_MULTICAST_LOOP = IP_MULTICAST_LOOP
        self.IP_TOS = IP_TOS
        self.SO_BROADCAST = SO_BROADCAST
        self.SO_CONDITIONAL_ACCEPT = SO_CONDITIONAL_ACCEPT
        self.SO_KEEPALIVE = SO_KEEPALIVE
        self.SO_DONTROUTE = SO_DONTROUTE
        self.SO_LINGER = SO_LINGER
        self.SO_DONTLINGER = SO_DONTLINGER
        self.SO_OOBINLINE = SO_OOBINLINE
        self.SO_RCVBUF = SO_RCVBUF
        self.SO_GROUP_PRIORITY = SO_GROUP_PRIORITY
        self.SO_REUSEADDR = SO_REUSEADDR
        self.SO_DEBUG = SO_DEBUG
        self.SO_RCVTIMEO = SO_RCVTIMEO
        self.SO_SNDBUF = SO_SNDBUF
        self.SO_SNDTIMEO = SO_SNDTIMEO
        self.SO_UPDATE_ACCEPT_CONTEXT = SO_UPDATE_ACCEPT_CONTEXT
        self.SO_TIMEOUT = SO_TIMEOUT
        self.TCP_NODELAY = TCP_NODELAY
    def factory(*args_, **kwargs_):
        if SocketOptionsType.subclass:
            return SocketOptionsType.subclass(*args_, **kwargs_)
        else:
            return SocketOptionsType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_IP_MULTICAST_IF(self): return self.IP_MULTICAST_IF
    def set_IP_MULTICAST_IF(self, IP_MULTICAST_IF): self.IP_MULTICAST_IF = IP_MULTICAST_IF
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_IP_MULTICAST_IF2(self): return self.IP_MULTICAST_IF2
    def set_IP_MULTICAST_IF2(self, IP_MULTICAST_IF2): self.IP_MULTICAST_IF2 = IP_MULTICAST_IF2
    def get_IP_MULTICAST_LOOP(self): return self.IP_MULTICAST_LOOP
    def set_IP_MULTICAST_LOOP(self, IP_MULTICAST_LOOP): self.IP_MULTICAST_LOOP = IP_MULTICAST_LOOP
    def get_IP_TOS(self): return self.IP_TOS
    def set_IP_TOS(self, IP_TOS): self.IP_TOS = IP_TOS
    def get_SO_BROADCAST(self): return self.SO_BROADCAST
    def set_SO_BROADCAST(self, SO_BROADCAST): self.SO_BROADCAST = SO_BROADCAST
    def get_SO_CONDITIONAL_ACCEPT(self): return self.SO_CONDITIONAL_ACCEPT
    def set_SO_CONDITIONAL_ACCEPT(self, SO_CONDITIONAL_ACCEPT): self.SO_CONDITIONAL_ACCEPT = SO_CONDITIONAL_ACCEPT
    def get_SO_KEEPALIVE(self): return self.SO_KEEPALIVE
    def set_SO_KEEPALIVE(self, SO_KEEPALIVE): self.SO_KEEPALIVE = SO_KEEPALIVE
    def get_SO_DONTROUTE(self): return self.SO_DONTROUTE
    def set_SO_DONTROUTE(self, SO_DONTROUTE): self.SO_DONTROUTE = SO_DONTROUTE
    def get_SO_LINGER(self): return self.SO_LINGER
    def set_SO_LINGER(self, SO_LINGER): self.SO_LINGER = SO_LINGER
    def validate_UnsignedIntegerObjectPropertyType(self, value):
        # Validate type cybox_common.UnsignedIntegerObjectPropertyType, a restriction on None.
        pass
    def get_SO_DONTLINGER(self): return self.SO_DONTLINGER
    def set_SO_DONTLINGER(self, SO_DONTLINGER): self.SO_DONTLINGER = SO_DONTLINGER
    def get_SO_OOBINLINE(self): return self.SO_OOBINLINE
    def set_SO_OOBINLINE(self, SO_OOBINLINE): self.SO_OOBINLINE = SO_OOBINLINE
    def get_SO_RCVBUF(self): return self.SO_RCVBUF
    def set_SO_RCVBUF(self, SO_RCVBUF): self.SO_RCVBUF = SO_RCVBUF
    def get_SO_GROUP_PRIORITY(self): return self.SO_GROUP_PRIORITY
    def set_SO_GROUP_PRIORITY(self, SO_GROUP_PRIORITY): self.SO_GROUP_PRIORITY = SO_GROUP_PRIORITY
    def get_SO_REUSEADDR(self): return self.SO_REUSEADDR
    def set_SO_REUSEADDR(self, SO_REUSEADDR): self.SO_REUSEADDR = SO_REUSEADDR
    def get_SO_DEBUG(self): return self.SO_DEBUG
    def set_SO_DEBUG(self, SO_DEBUG): self.SO_DEBUG = SO_DEBUG
    def get_SO_RCVTIMEO(self): return self.SO_RCVTIMEO
    def set_SO_RCVTIMEO(self, SO_RCVTIMEO): self.SO_RCVTIMEO = SO_RCVTIMEO
    def get_SO_SNDBUF(self): return self.SO_SNDBUF
    def set_SO_SNDBUF(self, SO_SNDBUF): self.SO_SNDBUF = SO_SNDBUF
    def get_SO_SNDTIMEO(self): return self.SO_SNDTIMEO
    def set_SO_SNDTIMEO(self, SO_SNDTIMEO): self.SO_SNDTIMEO = SO_SNDTIMEO
    def get_SO_UPDATE_ACCEPT_CONTEXT(self): return self.SO_UPDATE_ACCEPT_CONTEXT
    def set_SO_UPDATE_ACCEPT_CONTEXT(self, SO_UPDATE_ACCEPT_CONTEXT): self.SO_UPDATE_ACCEPT_CONTEXT = SO_UPDATE_ACCEPT_CONTEXT
    def get_SO_TIMEOUT(self): return self.SO_TIMEOUT
    def set_SO_TIMEOUT(self, SO_TIMEOUT): self.SO_TIMEOUT = SO_TIMEOUT
    def get_TCP_NODELAY(self): return self.TCP_NODELAY
    def set_TCP_NODELAY(self, TCP_NODELAY): self.TCP_NODELAY = TCP_NODELAY
    def hasContent_(self):
        if (
            self.IP_MULTICAST_IF is not None or
            self.IP_MULTICAST_IF2 is not None or
            self.IP_MULTICAST_LOOP is not None or
            self.IP_TOS is not None or
            self.SO_BROADCAST is not None or
            self.SO_CONDITIONAL_ACCEPT is not None or
            self.SO_KEEPALIVE is not None or
            self.SO_DONTROUTE is not None or
            self.SO_LINGER is not None or
            self.SO_DONTLINGER is not None or
            self.SO_OOBINLINE is not None or
            self.SO_RCVBUF is not None or
            self.SO_GROUP_PRIORITY is not None or
            self.SO_REUSEADDR is not None or
            self.SO_DEBUG is not None or
            self.SO_RCVTIMEO is not None or
            self.SO_SNDBUF is not None or
            self.SO_SNDTIMEO is not None or
            self.SO_UPDATE_ACCEPT_CONTEXT is not None or
            self.SO_TIMEOUT is not None or
            self.TCP_NODELAY is not None
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='NetworkSocketObj:', name_='SocketOptionsType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='SocketOptionsType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='NetworkSocketObj:', name_='SocketOptionsType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='NetworkSocketObj:', name_='SocketOptionsType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.IP_MULTICAST_IF is not None:
            self.IP_MULTICAST_IF.export(lwrite, level, 'NetworkSocketObj:', name_='IP_MULTICAST_IF', pretty_print=pretty_print)
        if self.IP_MULTICAST_IF2 is not None:
            self.IP_MULTICAST_IF2.export(lwrite, level, 'NetworkSocketObj:', name_='IP_MULTICAST_IF2', pretty_print=pretty_print)
        if self.IP_MULTICAST_LOOP is not None:
            showIndent(lwrite, level, pretty_print)
            lwrite('<%sIP_MULTICAST_LOOP>%s</%sIP_MULTICAST_LOOP>%s' % ('NetworkSocketObj:', self.gds_format_boolean(self.IP_MULTICAST_LOOP, input_name='IP_MULTICAST_LOOP'), 'NetworkSocketObj:', eol_))
        if self.IP_TOS is not None:
            self.IP_TOS.export(lwrite, level, 'NetworkSocketObj:', name_='IP_TOS', pretty_print=pretty_print)
        if self.SO_BROADCAST is not None:
            showIndent(lwrite, level, pretty_print)
            lwrite('<%sSO_BROADCAST>%s</%sSO_BROADCAST>%s' % ('NetworkSocketObj:', self.gds_format_boolean(self.SO_BROADCAST, input_name='SO_BROADCAST'), 'NetworkSocketObj:', eol_))
        if self.SO_CONDITIONAL_ACCEPT is not None:
            showIndent(lwrite, level, pretty_print)
            lwrite('<%sSO_CONDITIONAL_ACCEPT>%s</%sSO_CONDITIONAL_ACCEPT>%s' % ('NetworkSocketObj:', self.gds_format_boolean(self.SO_CONDITIONAL_ACCEPT, input_name='SO_CONDITIONAL_ACCEPT'), 'NetworkSocketObj:', eol_))
        if self.SO_KEEPALIVE is not None:
            showIndent(lwrite, level, pretty_print)
            lwrite('<%sSO_KEEPALIVE>%s</%sSO_KEEPALIVE>%s' % ('NetworkSocketObj:', self.gds_format_boolean(self.SO_KEEPALIVE, input_name='SO_KEEPALIVE'), 'NetworkSocketObj:', eol_))
        if self.SO_DONTROUTE is not None:
            showIndent(lwrite, level, pretty_print)
            lwrite('<%sSO_DONTROUTE>%s</%sSO_DONTROUTE>%s' % ('NetworkSocketObj:', self.gds_format_boolean(self.SO_DONTROUTE, input_name='SO_DONTROUTE'), 'NetworkSocketObj:', eol_))
        if self.SO_LINGER is not None:
            self.SO_LINGER.export(lwrite, level, 'NetworkSocketObj:', name_='SO_LINGER', pretty_print=pretty_print)
        if self.SO_DONTLINGER is not None:
            showIndent(lwrite, level, pretty_print)
            lwrite('<%sSO_DONTLINGER>%s</%sSO_DONTLINGER>%s' % ('NetworkSocketObj:', self.gds_format_boolean(self.SO_DONTLINGER, input_name='SO_DONTLINGER'), 'NetworkSocketObj:', eol_))
        if self.SO_OOBINLINE is not None:
            showIndent(lwrite, level, pretty_print)
            lwrite('<%sSO_OOBINLINE>%s</%sSO_OOBINLINE>%s' % ('NetworkSocketObj:', self.gds_format_boolean(self.SO_OOBINLINE, input_name='SO_OOBINLINE'), 'NetworkSocketObj:', eol_))
        if self.SO_RCVBUF is not None:
            self.SO_RCVBUF.export(lwrite, level, 'NetworkSocketObj:', name_='SO_RCVBUF', pretty_print=pretty_print)
        if self.SO_GROUP_PRIORITY is not None:
            self.SO_GROUP_PRIORITY.export(lwrite, level, 'NetworkSocketObj:', name_='SO_GROUP_PRIORITY', pretty_print=pretty_print)
        if self.SO_REUSEADDR is not None:
            showIndent(lwrite, level, pretty_print)
            lwrite('<%sSO_REUSEADDR>%s</%sSO_REUSEADDR>%s' % ('NetworkSocketObj:', self.gds_format_boolean(self.SO_REUSEADDR, input_name='SO_REUSEADDR'), 'NetworkSocketObj:', eol_))
        if self.SO_DEBUG is not None:
            showIndent(lwrite, level, pretty_print)
            lwrite('<%sSO_DEBUG>%s</%sSO_DEBUG>%s' % ('NetworkSocketObj:', self.gds_format_boolean(self.SO_DEBUG, input_name='SO_DEBUG'), 'NetworkSocketObj:', eol_))
        if self.SO_RCVTIMEO is not None:
            self.SO_RCVTIMEO.export(lwrite, level, 'NetworkSocketObj:', name_='SO_RCVTIMEO', pretty_print=pretty_print)
        if self.SO_SNDBUF is not None:
            self.SO_SNDBUF.export(lwrite, level, 'NetworkSocketObj:', name_='SO_SNDBUF', pretty_print=pretty_print)
        if self.SO_SNDTIMEO is not None:
            self.SO_SNDTIMEO.export(lwrite, level, 'NetworkSocketObj:', name_='SO_SNDTIMEO', pretty_print=pretty_print)
        if self.SO_UPDATE_ACCEPT_CONTEXT is not None:
            self.SO_UPDATE_ACCEPT_CONTEXT.export(lwrite, level, 'NetworkSocketObj:', name_='SO_UPDATE_ACCEPT_CONTEXT', pretty_print=pretty_print)
        if self.SO_TIMEOUT is not None:
            self.SO_TIMEOUT.export(lwrite, level, 'NetworkSocketObj:', name_='SO_TIMEOUT', pretty_print=pretty_print)
        if self.TCP_NODELAY is not None:
            showIndent(lwrite, level, pretty_print)
            lwrite('<%sTCP_NODELAY>%s</%sTCP_NODELAY>%s' % ('NetworkSocketObj:', self.gds_format_boolean(self.TCP_NODELAY, input_name='TCP_NODELAY'), 'NetworkSocketObj:', eol_))
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'IP_MULTICAST_IF':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_IP_MULTICAST_IF(obj_)
        elif nodeName_ == 'IP_MULTICAST_IF2':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_IP_MULTICAST_IF2(obj_)
        elif nodeName_ == 'IP_MULTICAST_LOOP':
            sval_ = child_.text
            if sval_ in ('true', '1'):
                ival_ = True
            elif sval_ in ('false', '0'):
                ival_ = False
            else:
                raise_parse_error(child_, 'requires boolean')
            ival_ = self.gds_validate_boolean(ival_, node, 'IP_MULTICAST_LOOP')
            self.IP_MULTICAST_LOOP = ival_
        elif nodeName_ == 'IP_TOS':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_IP_TOS(obj_)
        elif nodeName_ == 'SO_BROADCAST':
            sval_ = child_.text
            if sval_ in ('true', '1'):
                ival_ = True
            elif sval_ in ('false', '0'):
                ival_ = False
            else:
                raise_parse_error(child_, 'requires boolean')
            ival_ = self.gds_validate_boolean(ival_, node, 'SO_BROADCAST')
            self.SO_BROADCAST = ival_
        elif nodeName_ == 'SO_CONDITIONAL_ACCEPT':
            sval_ = child_.text
            if sval_ in ('true', '1'):
                ival_ = True
            elif sval_ in ('false', '0'):
                ival_ = False
            else:
                raise_parse_error(child_, 'requires boolean')
            ival_ = self.gds_validate_boolean(ival_, node, 'SO_CONDITIONAL_ACCEPT')
            self.SO_CONDITIONAL_ACCEPT = ival_
        elif nodeName_ == 'SO_KEEPALIVE':
            sval_ = child_.text
            if sval_ in ('true', '1'):
                ival_ = True
            elif sval_ in ('false', '0'):
                ival_ = False
            else:
                raise_parse_error(child_, 'requires boolean')
            ival_ = self.gds_validate_boolean(ival_, node, 'SO_KEEPALIVE')
            self.SO_KEEPALIVE = ival_
        elif nodeName_ == 'SO_DONTROUTE':
            sval_ = child_.text
            if sval_ in ('true', '1'):
                ival_ = True
            elif sval_ in ('false', '0'):
                ival_ = False
            else:
                raise_parse_error(child_, 'requires boolean')
            ival_ = self.gds_validate_boolean(ival_, node, 'SO_DONTROUTE')
            self.SO_DONTROUTE = ival_
        elif nodeName_ == 'SO_LINGER':
            obj_ = cybox_common.UnsignedIntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_SO_LINGER(obj_)
        elif nodeName_ == 'SO_DONTLINGER':
            sval_ = child_.text
            if sval_ in ('true', '1'):
                ival_ = True
            elif sval_ in ('false', '0'):
                ival_ = False
            else:
                raise_parse_error(child_, 'requires boolean')
            ival_ = self.gds_validate_boolean(ival_, node, 'SO_DONTLINGER')
            self.SO_DONTLINGER = ival_
        elif nodeName_ == 'SO_OOBINLINE':
            sval_ = child_.text
            if sval_ in ('true', '1'):
                ival_ = True
            elif sval_ in ('false', '0'):
                ival_ = False
            else:
                raise_parse_error(child_, 'requires boolean')
            ival_ = self.gds_validate_boolean(ival_, node, 'SO_OOBINLINE')
            self.SO_OOBINLINE = ival_
        elif nodeName_ == 'SO_RCVBUF':
            obj_ = cybox_common.UnsignedIntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_SO_RCVBUF(obj_)
        elif nodeName_ == 'SO_GROUP_PRIORITY':
            obj_ = cybox_common.UnsignedIntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_SO_GROUP_PRIORITY(obj_)
        elif nodeName_ == 'SO_REUSEADDR':
            sval_ = child_.text
            if sval_ in ('true', '1'):
                ival_ = True
            elif sval_ in ('false', '0'):
                ival_ = False
            else:
                raise_parse_error(child_, 'requires boolean')
            ival_ = self.gds_validate_boolean(ival_, node, 'SO_REUSEADDR')
            self.SO_REUSEADDR = ival_
        elif nodeName_ == 'SO_DEBUG':
            sval_ = child_.text
            if sval_ in ('true', '1'):
                ival_ = True
            elif sval_ in ('false', '0'):
                ival_ = False
            else:
                raise_parse_error(child_, 'requires boolean')
            ival_ = self.gds_validate_boolean(ival_, node, 'SO_DEBUG')
            self.SO_DEBUG = ival_
        elif nodeName_ == 'SO_RCVTIMEO':
            obj_ = cybox_common.UnsignedIntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_SO_RCVTIMEO(obj_)
        elif nodeName_ == 'SO_SNDBUF':
            obj_ = cybox_common.UnsignedIntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_SO_SNDBUF(obj_)
        elif nodeName_ == 'SO_SNDTIMEO':
            obj_ = cybox_common.UnsignedIntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_SO_SNDTIMEO(obj_)
        elif nodeName_ == 'SO_UPDATE_ACCEPT_CONTEXT':
            obj_ = cybox_common.UnsignedIntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_SO_UPDATE_ACCEPT_CONTEXT(obj_)
        elif nodeName_ == 'SO_TIMEOUT':
            obj_ = cybox_common.UnsignedIntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_SO_TIMEOUT(obj_)
        elif nodeName_ == 'TCP_NODELAY':
            sval_ = child_.text
            if sval_ in ('true', '1'):
                ival_ = True
            elif sval_ in ('false', '0'):
                ival_ = False
            else:
                raise_parse_error(child_, 'requires boolean')
            ival_ = self.gds_validate_boolean(ival_, node, 'TCP_NODELAY')
            self.TCP_NODELAY = ival_
# end class SocketOptionsType

class ProtocolType(cybox_common.BaseObjectPropertyType):
    """ProtocolType specifies protocol types, via a union of the
    ProtocolTypeEnum type and the atomic xs:string type. Its base
    type is the CybOX Core cybox_common.BaseObjectPropertyType, for permitting
    complex (i.e. regular-expression based) specifications.This
    attribute is optional and specifies the expected type for the
    value of the specified property."""

    subclass = None
    superclass = cybox_common.BaseObjectPropertyType
    def __init__(self, obfuscation_algorithm_ref=None, refanging_transform_type=None, has_changed=None, delimiter='##comma##', pattern_type=None, datatype='string', refanging_transform=None, is_case_sensitive=True, bit_mask=None, appears_random=None, observed_encoding=None, defanging_algorithm_ref=None, is_obfuscated=None, regex_syntax=None, apply_condition='ANY', trend=None, idref=None, is_defanged=None, id=None, condition=None, valueOf_=None):
        super(ProtocolType, self).__init__(obfuscation_algorithm_ref, refanging_transform_type, has_changed, delimiter, pattern_type, datatype, refanging_transform, is_case_sensitive, bit_mask, appears_random, observed_encoding, defanging_algorithm_ref, is_obfuscated, regex_syntax, apply_condition, trend, idref, is_defanged, id, condition, valueOf_)
        self.datatype = _cast(None, datatype)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if ProtocolType.subclass:
            return ProtocolType.subclass(*args_, **kwargs_)
        else:
            return ProtocolType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_datatype(self): return self.datatype
    def set_datatype(self, datatype): self.datatype = datatype
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_ or
            super(ProtocolType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='NetworkSocketObj:', name_='ProtocolType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='ProtocolType')
        if self.hasContent_():
            lwrite('>')
            lwrite(quote_xml(self.valueOf_))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='NetworkSocketObj:', name_='ProtocolType'):
        super(ProtocolType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='ProtocolType')
        if self.datatype is not None:

            lwrite(' datatype=%s' % (quote_attrib(self.datatype), ))
    def exportChildren(self, lwrite, level, namespace_='NetworkSocketObj:', name_='ProtocolType', fromsubclass_=False, pretty_print=True):
        super(ProtocolType, self).exportChildren(lwrite, level, 'NetworkSocketObj:', name_, True, pretty_print=pretty_print)
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('datatype', node)
        if value is not None:

            self.datatype = value
        super(ProtocolType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class ProtocolType

class SocketType(cybox_common.BaseObjectPropertyType):
    """SocketType specifies socket types, via a union of the SocketTypeEnum
    type and the atomic xs:string type. Its base type is the CybOX
    Core cybox_common.BaseObjectPropertyType, for permitting complex (i.e.
    regular-expression based) specifications.This attribute is
    optional and specifies the expected type for the value of the
    specified property."""

    subclass = None
    superclass = cybox_common.BaseObjectPropertyType
    def __init__(self, obfuscation_algorithm_ref=None, refanging_transform_type=None, has_changed=None, delimiter='##comma##', pattern_type=None, datatype='string', refanging_transform=None, is_case_sensitive=True, bit_mask=None, appears_random=None, observed_encoding=None, defanging_algorithm_ref=None, is_obfuscated=None, regex_syntax=None, apply_condition='ANY', trend=None, idref=None, is_defanged=None, id=None, condition=None, valueOf_=None):
        super(SocketType, self).__init__(obfuscation_algorithm_ref, refanging_transform_type, has_changed, delimiter, pattern_type, datatype, refanging_transform, is_case_sensitive, bit_mask, appears_random, observed_encoding, defanging_algorithm_ref, is_obfuscated, regex_syntax, apply_condition, trend, idref, is_defanged, id, condition, valueOf_)
        self.datatype = _cast(None, datatype)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if SocketType.subclass:
            return SocketType.subclass(*args_, **kwargs_)
        else:
            return SocketType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_datatype(self): return self.datatype
    def set_datatype(self, datatype): self.datatype = datatype
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_ or
            super(SocketType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='NetworkSocketObj:', name_='SocketType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='SocketType')
        if self.hasContent_():
            lwrite('>')
            lwrite(quote_xml(self.valueOf_))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='NetworkSocketObj:', name_='SocketType'):
        super(SocketType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='SocketType')
        if self.datatype is not None:

            lwrite(' datatype=%s' % (quote_attrib(self.datatype), ))
    def exportChildren(self, lwrite, level, namespace_='NetworkSocketObj:', name_='SocketType', fromsubclass_=False, pretty_print=True):
        super(SocketType, self).exportChildren(lwrite, level, 'NetworkSocketObj:', name_, True, pretty_print=pretty_print)
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('datatype', node)
        if value is not None:

            self.datatype = value
        super(SocketType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class SocketType

class DomainFamilyType(cybox_common.BaseObjectPropertyType):
    """DomainFamilyType specifies domain family types, via a union of the
    DomainTypeEnum type and the atomic xs:string type. Its base type
    is the CybOX Core cybox_common.BaseObjectPropertyType, for permitting complex
    (i.e. regular-expression based) specifications.This attribute is
    optional and specifies the expected type for the value of the
    specified property."""

    subclass = None
    superclass = cybox_common.BaseObjectPropertyType
    def __init__(self, obfuscation_algorithm_ref=None, refanging_transform_type=None, has_changed=None, delimiter='##comma##', pattern_type=None, datatype='string', refanging_transform=None, is_case_sensitive=True, bit_mask=None, appears_random=None, observed_encoding=None, defanging_algorithm_ref=None, is_obfuscated=None, regex_syntax=None, apply_condition='ANY', trend=None, idref=None, is_defanged=None, id=None, condition=None, valueOf_=None):
        super(DomainFamilyType, self).__init__(obfuscation_algorithm_ref, refanging_transform_type, has_changed, delimiter, pattern_type, datatype, refanging_transform, is_case_sensitive, bit_mask, appears_random, observed_encoding, defanging_algorithm_ref, is_obfuscated, regex_syntax, apply_condition, trend, idref, is_defanged, id, condition, valueOf_)
        self.datatype = _cast(None, datatype)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if DomainFamilyType.subclass:
            return DomainFamilyType.subclass(*args_, **kwargs_)
        else:
            return DomainFamilyType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_datatype(self): return self.datatype
    def set_datatype(self, datatype): self.datatype = datatype
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_ or
            super(DomainFamilyType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='NetworkSocketObj:', name_='DomainFamilyType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='DomainFamilyType')
        if self.hasContent_():
            lwrite('>')
            lwrite(quote_xml(self.valueOf_))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='NetworkSocketObj:', name_='DomainFamilyType'):
        super(DomainFamilyType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='DomainFamilyType')
        if self.datatype is not None:

            lwrite(' datatype=%s' % (quote_attrib(self.datatype), ))
    def exportChildren(self, lwrite, level, namespace_='NetworkSocketObj:', name_='DomainFamilyType', fromsubclass_=False, pretty_print=True):
        super(DomainFamilyType, self).exportChildren(lwrite, level, 'NetworkSocketObj:', name_, True, pretty_print=pretty_print)
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('datatype', node)
        if value is not None:

            self.datatype = value
        super(DomainFamilyType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class DomainFamilyType

class AddressFamilyType(cybox_common.BaseObjectPropertyType):
    """AddressFamilyType specifies address family types, via a union of the
    AddressFamilyTypeEnum type and the atomic xs:string type. Its
    base type is the CybOX Core cybox_common.BaseObjectPropertyType, for
    permitting complex (i.e. regular-expression based)
    specifications.This attribute is optional and specifies the
    expected type for the value of the specified property."""

    subclass = None
    superclass = cybox_common.BaseObjectPropertyType
    def __init__(self, obfuscation_algorithm_ref=None, refanging_transform_type=None, has_changed=None, delimiter='##comma##', pattern_type=None, datatype='string', refanging_transform=None, is_case_sensitive=True, bit_mask=None, appears_random=None, observed_encoding=None, defanging_algorithm_ref=None, is_obfuscated=None, regex_syntax=None, apply_condition='ANY', trend=None, idref=None, is_defanged=None, id=None, condition=None, valueOf_=None):
        super(AddressFamilyType, self).__init__(obfuscation_algorithm_ref, refanging_transform_type, has_changed, delimiter, pattern_type, datatype, refanging_transform, is_case_sensitive, bit_mask, appears_random, observed_encoding, defanging_algorithm_ref, is_obfuscated, regex_syntax, apply_condition, trend, idref, is_defanged, id, condition, valueOf_)
        self.datatype = _cast(None, datatype)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if AddressFamilyType.subclass:
            return AddressFamilyType.subclass(*args_, **kwargs_)
        else:
            return AddressFamilyType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_datatype(self): return self.datatype
    def set_datatype(self, datatype): self.datatype = datatype
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_ or
            super(AddressFamilyType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='NetworkSocketObj:', name_='AddressFamilyType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='AddressFamilyType')
        if self.hasContent_():
            lwrite('>')
            lwrite(quote_xml(self.valueOf_))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='NetworkSocketObj:', name_='AddressFamilyType'):
        super(AddressFamilyType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='AddressFamilyType')
        if self.datatype is not None:

            lwrite(' datatype=%s' % (quote_attrib(self.datatype), ))
    def exportChildren(self, lwrite, level, namespace_='NetworkSocketObj:', name_='AddressFamilyType', fromsubclass_=False, pretty_print=True):
        super(AddressFamilyType, self).exportChildren(lwrite, level, 'NetworkSocketObj:', name_, True, pretty_print=pretty_print)
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('datatype', node)
        if value is not None:

            self.datatype = value
        super(AddressFamilyType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class AddressFamilyType

class NetworkSocketObjectType(cybox_common.ObjectPropertiesType):
    """The NetworkSocketObjectType is intended to characterize network
    sockets.The is_blocking field specifies whether or not the
    socket is in blocking mode. The is_listening field specifies
    whether or not the socket is in listening mode."""

    subclass = None
    superclass = cybox_common.ObjectPropertiesType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, is_blocking=None, is_listening=None, Address_Family=None, Domain=None, Local_Address=None, Options=None, Protocol=None, Remote_Address=None, Type=None, Socket_Descriptor=None):
        super(NetworkSocketObjectType, self).__init__(object_reference, Custom_Properties, xsi_type )
        self.is_blocking = _cast(bool, is_blocking)
        self.is_listening = _cast(bool, is_listening)
        self.Address_Family = Address_Family
        self.Domain = Domain
        self.Local_Address = Local_Address
        self.Options = Options
        self.Protocol = Protocol
        self.Remote_Address = Remote_Address
        self.Type = Type
        self.Socket_Descriptor = Socket_Descriptor
    def factory(*args_, **kwargs_):
        if NetworkSocketObjectType.subclass:
            return NetworkSocketObjectType.subclass(*args_, **kwargs_)
        else:
            return NetworkSocketObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Address_Family(self): return self.Address_Family
    def set_Address_Family(self, Address_Family): self.Address_Family = Address_Family
    def validate_AddressFamilyType(self, value):
        # Validate type AddressFamilyType, a restriction on None.
        pass
    def get_Domain(self): return self.Domain
    def set_Domain(self, Domain): self.Domain = Domain
    def validate_DomainFamilyType(self, value):
        # Validate type DomainFamilyType, a restriction on None.
        pass
    def get_Local_Address(self): return self.Local_Address
    def set_Local_Address(self, Local_Address): self.Local_Address = Local_Address
    def get_Options(self): return self.Options
    def set_Options(self, Options): self.Options = Options
    def get_Protocol(self): return self.Protocol
    def set_Protocol(self, Protocol): self.Protocol = Protocol
    def validate_ProtocolType(self, value):
        # Validate type ProtocolType, a restriction on None.
        pass
    def get_Remote_Address(self): return self.Remote_Address
    def set_Remote_Address(self, Remote_Address): self.Remote_Address = Remote_Address
    def get_Type(self): return self.Type
    def set_Type(self, Type): self.Type = Type
    def get_Socket_Descriptor(self): return self.Socket_Descriptor
    def set_Socket_Descriptor(self, Socket_Descriptor): self.Socket_Descriptor = Socket_Descriptor
    def validate_SocketType(self, value):
        # Validate type SocketType, a restriction on None.
        pass
    def get_is_blocking(self): return self.is_blocking
    def set_is_blocking(self, is_blocking): self.is_blocking = is_blocking
    def get_is_listening(self): return self.is_listening
    def set_is_listening(self, is_listening): self.is_listening = is_listening
    def hasContent_(self):
        if (
            self.Address_Family is not None or
            self.Domain is not None or
            self.Local_Address is not None or
            self.Options is not None or
            self.Protocol is not None or
            self.Remote_Address is not None or
            self.Type is not None or
            self.Socket_Descriptor is not None or
            super(NetworkSocketObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='NetworkSocketObj:', name_='NetworkSocketObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='NetworkSocketObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='NetworkSocketObj:', name_='NetworkSocketObjectType'):
        super(NetworkSocketObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='NetworkSocketObjectType')
        if self.is_blocking is not None:

            lwrite(' is_blocking="%s"' % self.gds_format_boolean(self.is_blocking, input_name='is_blocking'))
        if self.is_listening is not None:

            lwrite(' is_listening="%s"' % self.gds_format_boolean(self.is_listening, input_name='is_listening'))
    def exportChildren(self, lwrite, level, namespace_='NetworkSocketObj:', name_='NetworkSocketObjectType', fromsubclass_=False, pretty_print=True):
        super(NetworkSocketObjectType, self).exportChildren(lwrite, level, 'NetworkSocketObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Address_Family is not None:
            self.Address_Family.export(lwrite, level, 'NetworkSocketObj:', name_='Address_Family', pretty_print=pretty_print)
        if self.Domain is not None:
            self.Domain.export(lwrite, level, 'NetworkSocketObj:', name_='Domain', pretty_print=pretty_print)
        if self.Local_Address is not None:
            self.Local_Address.export(lwrite, level, 'NetworkSocketObj:', name_='Local_Address', pretty_print=pretty_print)
        if self.Options is not None:
            self.Options.export(lwrite, level, 'NetworkSocketObj:', name_='Options', pretty_print=pretty_print)
        if self.Protocol is not None:
            self.Protocol.export(lwrite, level, 'NetworkSocketObj:', name_='Protocol', pretty_print=pretty_print)
        if self.Remote_Address is not None:
            self.Remote_Address.export(lwrite, level, 'NetworkSocketObj:', name_='Remote_Address', pretty_print=pretty_print)
        if self.Type is not None:
            self.Type.export(lwrite, level, 'NetworkSocketObj:', name_='Type', pretty_print=pretty_print)
        if self.Socket_Descriptor is not None:
            self.Socket_Descriptor.export(lwrite, level, 'NetworkSocketObj:', name_='Socket_Descriptor', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('is_blocking', node)
        if value is not None:

            if value in ('true', '1'):
                self.is_blocking = True
            elif value in ('false', '0'):
                self.is_blocking = False
            else:
                raise_parse_error(node, 'Bad boolean attribute')
        value = find_attr_value_('is_listening', node)
        if value is not None:

            if value in ('true', '1'):
                self.is_listening = True
            elif value in ('false', '0'):
                self.is_listening = False
            else:
                raise_parse_error(node, 'Bad boolean attribute')
        super(NetworkSocketObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Address_Family':
            obj_ = AddressFamilyType.factory()
            obj_.build(child_)
            self.set_Address_Family(obj_)
        elif nodeName_ == 'Domain':
            obj_ = DomainFamilyType.factory()
            obj_.build(child_)
            self.set_Domain(obj_)
        elif nodeName_ == 'Local_Address':
            obj_ = socket_address_object.SocketAddressObjectType.factory()
            obj_.build(child_)
            self.set_Local_Address(obj_)
        elif nodeName_ == 'Options':
            obj_ = SocketOptionsType.factory()
            obj_.build(child_)
            self.set_Options(obj_)
        elif nodeName_ == 'Protocol':
            obj_ = ProtocolType.factory()
            obj_.build(child_)
            self.set_Protocol(obj_)
        elif nodeName_ == 'Remote_Address':
            obj_ = socket_address_object.SocketAddressObjectType.factory()
            obj_.build(child_)
            self.set_Remote_Address(obj_)
        elif nodeName_ == 'Type':
            obj_ = SocketType.factory()
            obj_.build(child_)
            self.set_Type(obj_)
        elif nodeName_ == 'Socket_Descriptor':
            obj_ = cybox_common.NonNegativeIntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Socket_Descriptor(obj_)
        super(NetworkSocketObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class NetworkSocketObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Errors': cybox_common.ErrorsType,
    'SO_GROUP_PRIORITY': cybox_common.UnsignedIntegerObjectPropertyType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'SO_RCVTIMEO': cybox_common.UnsignedIntegerObjectPropertyType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'SO_RCVBUF': cybox_common.UnsignedIntegerObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'SO_UPDATE_ACCEPT_CONTEXT': cybox_common.UnsignedIntegerObjectPropertyType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'SubDatum': cybox_common.MetadataType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Remote_Address': socket_address_object.SocketAddressObjectType,
    'Value': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'SO_LINGER': cybox_common.UnsignedIntegerObjectPropertyType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'SO_TIMEOUT': cybox_common.UnsignedIntegerObjectPropertyType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Functions': cybox_common.FunctionsType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'System': cybox_common.ObjectPropertiesType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Type': cybox_common.ControlledVocabularyStringType,
    'Compilers': cybox_common.CompilersType,
    'SO_SNDBUF': cybox_common.UnsignedIntegerObjectPropertyType,
    'String': cybox_common.ExtractedStringType,
    'Tool': cybox_common.ToolInformationType,
    'Build_Information': cybox_common.BuildInformationType,
    'Tool_Hashes': cybox_common.HashListType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'IP_TOS': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'Contributors': cybox_common.PersonnelType,
    'Reference_Description': cybox_common.StructuredTextType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Import': cybox_common.StringObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Dependencies': cybox_common.DependenciesType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateRangeType,
    'Hashes': cybox_common.HashListType,
    'Segments': cybox_common.HashSegmentsType,
    'Language': cybox_common.StringObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Error': cybox_common.ErrorType,
    'SO_SNDTIMEO': cybox_common.UnsignedIntegerObjectPropertyType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'IP_MULTICAST_IF2': cybox_common.StringObjectPropertyType,
    'Imports': cybox_common.ImportsType,
    'Library': cybox_common.LibraryType,
    'References': cybox_common.ToolReferencesType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'Time': cybox_common.TimeType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Libraries': cybox_common.LibrariesType,
    'Function': cybox_common.StringObjectPropertyType,
    'Description': cybox_common.StructuredTextType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'VLAN_Name': cybox_common.StringObjectPropertyType,
    'Socket_Address': socket_address_object.SocketAddressObjectType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'Port_Value': cybox_common.PositiveIntegerObjectPropertyType,
    'Compiler': cybox_common.CompilerType,
    'Name': cybox_common.StringObjectPropertyType,
    'Address_Value': cybox_common.StringObjectPropertyType,
    'VLAN_Num': cybox_common.IntegerObjectPropertyType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'IP_MULTICAST_IF': cybox_common.StringObjectPropertyType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Data_Size': cybox_common.DataSizeType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'Contributor': cybox_common.ContributorType,
    'Local_Address': socket_address_object.SocketAddressObjectType,
    'Tools': cybox_common.ToolsInformationType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Network_Socket'
        rootClass = NetworkSocketObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_=rootTag,
#        namespacedef_='',
#        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Network_Socket'
        rootClass = NetworkSocketObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Network_Socket'
        rootClass = NetworkSocketObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_="Network_Socket",
#        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "NetworkSocketObjectType",
    "SocketOptionsType",
    "AddressFamilyType",
    "DomainFamilyType",
    "SocketType",
    "ProtocolType"
    ]



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common
import cybox.objects


class ProductObjectType(cybox_common.ObjectPropertiesType):
    """The ProductObjectType type is intended to characterize software or
    hardware products."""

    subclass = None
    superclass = cybox_common.ObjectPropertiesType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, Edition=None, Language=None, Product=None, Update=None, Vendor=None, Version=None, Device_Details=None):
        super(ProductObjectType, self).__init__(object_reference, Custom_Properties, xsi_type )
        self.Edition = Edition
        self.Language = Language
        self.Product = Product
        self.Update = Update
        self.Vendor = Vendor
        self.Version = Version
        self.Device_Details = Device_Details
    def factory(*args_, **kwargs_):
        if ProductObjectType.subclass:
            return ProductObjectType.subclass(*args_, **kwargs_)
        else:
            return ProductObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Edition(self): return self.Edition
    def set_Edition(self, Edition): self.Edition = Edition
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_Language(self): return self.Language
    def set_Language(self, Language): self.Language = Language
    def get_Product(self): return self.Product
    def set_Product(self, Product): self.Product = Product
    def get_Update(self): return self.Update
    def set_Update(self, Update): self.Update = Update
    def get_Vendor(self): return self.Vendor
    def set_Vendor(self, Vendor): self.Vendor = Vendor
    def get_Version(self): return self.Version
    def set_Version(self, Version): self.Version = Version
    def get_Device_Details(self): return self.Device_Details
    def set_Device_Details(self, Device_Details): self.Device_Details = Device_Details
    def hasContent_(self):
        if (
            self.Edition is not None or
            self.Language is not None or
            self.Product is not None or
            self.Update is not None or
            self.Vendor is not None or
            self.Version is not None or
            self.Device_Details is not None or
            super(ProductObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='ProductObj:', name_='ProductObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='ProductObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='ProductObj:', name_='ProductObjectType'):
        super(ProductObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='ProductObjectType')
    def exportChildren(self, lwrite, level, namespace_='ProductObj:', name_='ProductObjectType', fromsubclass_=False, pretty_print=True):
        super(ProductObjectType, self).exportChildren(lwrite, level, 'ProductObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Edition is not None:
            self.Edition.export(lwrite, level, 'ProductObj:', name_='Edition', pretty_print=pretty_print)
        if self.Language is not None:
            self.Language.export(lwrite, level, 'ProductObj:', name_='Language', pretty_print=pretty_print)
        if self.Product is not None:
            self.Product.export(lwrite, level, 'ProductObj:', name_='Product', pretty_print=pretty_print)
        if self.Update is not None:
            self.Update.export(lwrite, level, 'ProductObj:', name_='Update', pretty_print=pretty_print)
        if self.Vendor is not None:
            self.Vendor.export(lwrite, level, 'ProductObj:', name_='Vendor', pretty_print=pretty_print)
        if self.Version is not None:
            self.Version.export(lwrite, level, 'ProductObj:', name_='Version', pretty_print=pretty_print)
        if self.Device_Details is not None:
            self.Device_Details.export(lwrite, level, 'ProductObj:', name_='Device_Details', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(ProductObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Edition':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Edition(obj_)
        elif nodeName_ == 'Language':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Language(obj_)
        elif nodeName_ == 'Product':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Product(obj_)
        elif nodeName_ == 'Update':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Update(obj_)
        elif nodeName_ == 'Vendor':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Vendor(obj_)
        elif nodeName_ == 'Version':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Version(obj_)
        elif nodeName_ == 'Device_Details':
            type_name_ = child_.attrib.get(
                '{http://www.w3.org/2001/XMLSchema-instance}type')
            if type_name_ is None:
                type_name_ = child_.attrib.get('type')
            if type_name_ is not None:
                type_names_ = type_name_.split(':')
                if len(type_names_) == 1:
                    type_name_ = type_names_[0]
                else:
                    type_name_ = type_names_[1]
                class_ = cybox.objects.get_class_for_object_type(type_name_)._binding_class
                obj_ = class_.factory()
                obj_.build(child_)
            else:
                raise NotImplementedError(
                    'Class not implemented for <Device_Details> element')
            self.set_Device_Details(obj_)

        super(ProductObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class ProductObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Errors': cybox_common.ErrorsType,
    'Time': cybox_common.TimeType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Update': cybox_common.StringObjectPropertyType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'SubDatum': cybox_common.MetadataType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Value': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Functions': cybox_common.FunctionsType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'System': cybox_common.ObjectPropertiesType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Version': cybox_common.StringObjectPropertyType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Type': cybox_common.ControlledVocabularyStringType,
    'Compilers': cybox_common.CompilersType,
    'Vendor': cybox_common.StringObjectPropertyType,
    'String': cybox_common.ExtractedStringType,
    'Tool': cybox_common.ToolInformationType,
    'Build_Information': cybox_common.BuildInformationType,
    'Tool_Hashes': cybox_common.HashListType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Language': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'Reference_Description': cybox_common.StructuredTextType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Import': cybox_common.StringObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Dependencies': cybox_common.DependenciesType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateRangeType,
    'Hashes': cybox_common.HashListType,
    'Segments': cybox_common.HashSegmentsType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Error': cybox_common.ErrorType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'Contributors': cybox_common.PersonnelType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'Imports': cybox_common.ImportsType,
    'Library': cybox_common.LibraryType,
    'Edition': cybox_common.StringObjectPropertyType,
    'References': cybox_common.ToolReferencesType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Libraries': cybox_common.LibrariesType,
    'Function': cybox_common.StringObjectPropertyType,
    'Product': cybox_common.StringObjectPropertyType,
    'Description': cybox_common.StructuredTextType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'Address': cybox_common.HexBinaryObjectPropertyType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'Compiler': cybox_common.CompilerType,
    'Name': cybox_common.StringObjectPropertyType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'Contributor': cybox_common.ContributorType,
    'Tools': cybox_common.ToolsInformationType,
    'Data_Size': cybox_common.DataSizeType,
    'Device_Details': cybox_common.ObjectPropertiesType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Product'
        rootClass = ProductObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_=rootTag,
#        namespacedef_='',
#        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Product'
        rootClass = ProductObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Product'
        rootClass = ProductObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_="Product",
#        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "ProductObjectType"
    ]



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields
from mixbox.vendor.six import u

import cybox.bindings.whois_object as whois_binding

from cybox.common import ObjectProperties, BaseProperty, String, DateTime, Date
from cybox.objects.address_object import Address, EmailAddress
from cybox.objects.uri_object import URI


class WhoisNameservers(entities.EntityList):
    _binding = whois_binding
    _binding_class = whois_binding.WhoisNameserversType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/whois-2"
    nameserver = fields.TypedField("Nameserver", URI, multiple=True)

class WhoisStatus(BaseProperty):
    _binding = whois_binding
    _binding_class = whois_binding.WhoisStatusType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/whois-2"
    datatype = "string"


class WhoisStatuses(entities.EntityList):
    _binding = whois_binding
    _binding_class = whois_binding.WhoisStatusesType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/whois-2"
    status = fields.TypedField("Status", WhoisStatus, multiple=True)

class WhoisContact(entities.Entity):
    _binding = whois_binding
    _binding_class = whois_binding.WhoisContactType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/whois-2"

    contact_type = fields.TypedField("contact_type")
    contact_id = fields.TypedField("Contact_ID", String)
    name = fields.TypedField("Name", String)
    address = fields.TypedField("Address", String)
    email_address = fields.TypedField("Email_Address", EmailAddress)
    phone_number = fields.TypedField("Phone_Number", String)
    fax_number = fields.TypedField("Fax_Number", String)
    organization = fields.TypedField("Organization", String)


class WhoisContacts(entities.EntityList):
    _binding = whois_binding
    _binding_class = whois_binding.WhoisContactsType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/whois-2"
    contact = fields.TypedField("Contact", WhoisContact, multiple=True)


class WhoisRegistrant(WhoisContact):
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/whois-2"
    _binding = whois_binding
    _binding_class = whois_binding.WhoisRegistrantInfoType

    registrant_id = fields.TypedField("Registrant_ID", String)


class WhoisRegistrants(entities.EntityList):
    _binding = whois_binding
    _binding_class = whois_binding.WhoisRegistrantsType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/whois-2"
    registrant = fields.TypedField("Registrant", WhoisRegistrant, multiple=True)


class WhoisRegistrar(entities.Entity):
    _binding = whois_binding
    _binding_class = whois_binding.WhoisRegistrarInfoType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/whois-2"

    registrar_id = fields.TypedField("Registrar_ID", String)
    registrar_guid = fields.TypedField("Registrar_GUID", String)
    name = fields.TypedField("Name", String)
    address = fields.TypedField("Address", String)
    email_address = fields.TypedField("Email_Address", EmailAddress)
    phone_number = fields.TypedField("Phone_Number", String)
    whois_server = fields.TypedField("Whois_Server", URI)
    referral_url = fields.TypedField("Referral_URL", URI)
    contacts = fields.TypedField("Contacts", WhoisContacts)


class WhoisEntry(ObjectProperties):
    _binding = whois_binding
    _binding_class = whois_binding.WhoisObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/whois-2"
    _XSI_NS = 'WhoisObj'
    _XSI_TYPE = 'WhoisObjectType'

    lookup_date = fields.TypedField("Lookup_Date", DateTime)
    remarks = fields.TypedField("Remarks", String)
    contact_info = fields.TypedField("Contact_Info", WhoisContact)
    domain_name = fields.TypedField("Domain_Name", URI)
    domain_id = fields.TypedField("Domain_ID", String)
    server_name = fields.TypedField("Server_Name", URI)
    ip_address = fields.TypedField("IP_Address", Address)
    dnssec = fields.TypedField("DNSSEC")
    nameservers = fields.TypedField("Nameservers", WhoisNameservers)
    status = fields.TypedField("Status", WhoisStatuses)
    updated_date = fields.TypedField("Updated_Date", Date)
    creation_date = fields.TypedField("Creation_Date", Date)
    expiration_date = fields.TypedField("Expiration_Date", Date)
    regional_internet_registry = fields.TypedField("Regional_Internet_Registry", String)
    sponsoring_registrar = fields.TypedField("Sponsoring_Registrar", String)
    registrar_info = fields.TypedField("Registrar_Info", WhoisRegistrar)
    registrants = fields.TypedField("Registrants", WhoisRegistrants)

    DNSSEC_SIGNED = u("Signed")
    DNSSEC_UNSIGNED = u("Unsigned")



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.
from mixbox import entities
from mixbox import fields
from mixbox import idgen

import cybox
import cybox.utils
import cybox.bindings.cybox_core as core_binding
from cybox.common.object_properties import ObjectPropertiesFactory, ObjectProperties
from cybox.common.vocabs import VocabField
from cybox.common.vocabs import ObjectRelationship as Relationship


_EXTERNAL_CLASSES = {}  # Maps xsi:type values to binding


def add_external_class(klass, xsi_type):
    """Adds a class implementation to this binding's globals() dict.

    These classes can be used to implement Properties,
    Domain_Specific_Object_Properties, or Defined_Effect fields on an Object.

    Args:
        klass (class): Python class that implements the new type
        xsi_type (str): An xsi:type value corresponding to the `klass`.
    """
    _EXTERNAL_CLASSES[xsi_type] = klass


class ExternalTypeFactory(entities.EntityFactory):
    def entity_class(cls, key):
        return _EXTERNAL_CLASSES[key]


def _modify_properties_parent(instance, value=None):
    if isinstance(instance, RelatedObject) and not instance._inline:
        return
    if instance.properties:
        instance.properties.parent = instance


def _cache_object(instance, value=None):
    if instance.id_:
        cybox.utils.cache_put(instance)


class Object(entities.Entity):
    """The CybOX Object element.

    Currently only supports the following data members:
    - id\_
    - idref
    - properties
    - related_objects
    - domain specific object properties
    """
    _binding = core_binding
    _binding_class = _binding.ObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/core-2'

    id_ = fields.IdField("id", postset_hook=_cache_object)
    idref = fields.IdrefField("idref")
    properties = fields.TypedField("Properties", ObjectProperties, factory=ObjectPropertiesFactory, postset_hook=_modify_properties_parent)
    related_objects = fields.TypedField("Related_Objects", "cybox.core.object.RelatedObjects")
    domain_specific_object_properties = fields.TypedField("Domain_Specific_Object_Properties", "cybox.core.DomainSpecificObjectProperties", factory=ExternalTypeFactory)

    def __init__(self, properties=None, type_=None, id_=None, idref=None):
        super(Object, self).__init__()

        if properties:
            prefix = str(properties.__class__.__name__)
        else:
            prefix = "Object"

        self.id_ = id_ or idgen.create_id(prefix=prefix)
        self.idref = idref
        self.properties = properties
        self.related_objects = RelatedObjects()

    def __str__(self):
        if self.id_ is not None:
            return self.id_
        elif self.idref is not None:
            return self.idref
        else:
            return super(Object, self).__repr__()
        
    def add_related(self, related, relationship, inline=True):
        if not isinstance(related, ObjectProperties):
            raise ValueError("Must be a ObjectProperties")
        r = RelatedObject(related, relationship=relationship, inline=inline)
        self.related_objects.append(r)


class RelatedObject(Object):
    _binding = core_binding
    _binding_class = _binding.RelatedObjectType

    relationship = VocabField("Relationship", Relationship)

    def __init__(self, *args, **kwargs):
        relationship = kwargs.pop('relationship', None)
        self._inline = kwargs.pop('inline', True)

        super(RelatedObject, self).__init__(*args, **kwargs)
        self.relationship = relationship

        if not self._inline and self.properties:
            self.idref = self.properties.parent.id_

    def __str__(self):
        return "Related: " + super(RelatedObject, self).__str__()

    #TODO: make this a property somehow
    def get_properties(self):
        if self.properties:
            return self.properties
        elif self.idref:
            return cybox.utils.cache_get(self.idref).properties
        else:
            return None

    def to_obj(self, ns_info=None):
        relobj_obj = super(RelatedObject, self).to_obj(ns_info=ns_info)

        if not self._inline:
            relobj_obj.idref = self.idref

        if self.relationship:
            relobj_obj.Relationship = self.relationship.to_obj(ns_info=ns_info)

        return relobj_obj

    def to_dict(self):
        if self._inline:
            return super(RelatedObject, self).to_dict()

        relobj_dict = {'idref': self.idref}

        if self.relationship:
            relobj_dict['relationship'] = self.relationship.to_dict()

        return relobj_dict

    @classmethod
    def from_obj(cls, cls_obj):
        if not cls_obj:
            return None

        relobj = super(RelatedObject, cls).from_obj(cls_obj)

        if not relobj.idref and relobj.properties:
            relobj._inline = True

        return relobj

    @classmethod
    def from_dict(cls, cls_dict):
        if not cls_dict:
            return None

        relobj = super(RelatedObject, cls).from_dict(cls_dict)

        if not relobj.idref and relobj.properties:
            relobj._inline = True

        return relobj


class RelatedObjects(entities.EntityList):
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/core-2"
    _binding = core_binding
    _binding_class = _binding.RelatedObjectsType
    related_object = fields.TypedField("Related_Object", RelatedObject, multiple=True)

class DomainSpecificObjectProperties(entities.Entity):
    """The Cybox DomainSpecificObjectProperties base class."""
    _binding = core_binding
    _binding_class = _binding.DomainSpecificObjectPropertiesType

    # Override in subclass
    _XSI_TYPE = None
    _XSI_NS = None

    def to_obj(self, ns_info=None):
        obj = super(DomainSpecificObjectProperties, self).to_obj(ns_info=ns_info)
        obj.xsi_type = "%s:%s" % (self._XSI_NS, self._XSI_TYPE)

    def to_dict(self):
        d = super(DomainSpecificObjectProperties, self).to_dict()
        d['xsi:type'] = self._XSI_TYPE
        return d





# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.gui_dialogbox_object as gui_dialogbox_binding
from cybox.common import String
from cybox.objects.gui_object import GUI


class GUIDialogbox(GUI):
    _binding = gui_dialogbox_binding
    _binding_class = gui_dialogbox_binding.GUIDialogboxObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/gui-dialogbox-2"
    _XSI_NS = "GUIDialogBoxObj"
    _XSI_TYPE = "GUIDialogboxObjectType"

    box_caption = fields.TypedField("Box_Caption", String)
    box_text = fields.TypedField("Box_Text", String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from cybox.objects.disk_partition_object import DiskPartition
from cybox.test.objects import ObjectTestCase


class TestDiskPartition(ObjectTestCase, unittest.TestCase):
    object_type = "DiskPartitionObjectType"
    klass = DiskPartition

    _full_dict = {
        'created': "2007-10-05T07:14:21+00:00",
        'device_name': u"C partition",
        'mount_point': u"C:",
        'partition_id': 8,
        'partition_length': 5000000,
        'partition_offset': 1000000,
        'space_left': 50000,
        'space_used': 50001,
        'total_space': 100001,
        'type': "PARTITION_FAT_12",
        'xsi:type': object_type,
    }

    # https://github.com/CybOXProject/python-cybox/issues/267
    def test_type(self):
        partition = DiskPartition()
        partition.type_ = "PARTITION_FAT32"
        self.assertTrue(b"PARTITION_FAT32" in partition.to_xml())


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from cybox.objects.api_object import API

from cybox.test import EntityTestCase, round_trip
from cybox.test.objects import ObjectTestCase


class TestAPI(ObjectTestCase, unittest.TestCase):
    object_type = "APIObjectType"
    klass = API

    _full_dict = {
        'description': "Some API",
        'function_name': "someTestFunction",
        'normalized_function_name': "some_test_function",
        'platform': {
            'description': "Test Platform"
        },
        'address': "876cdbeb87cd8e7",
        'xsi:type': object_type,
    }

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common


class KernelHookType(cybox_common.BaseObjectPropertyType):
    """KernelHookType specifies Windows kernel hook types via a union of
    the KernelHookTypeEnum type and the atomic xs:string type. Its
    base type is the CybOX Core cybox_common.BaseObjectPropertyType, for
    permitting complex (i.e. regular-expression based)
    specifications.This attribute is optional and specifies the
    expected type for the value of the specified property."""

    subclass = None
    superclass = cybox_common.BaseObjectPropertyType
    def __init__(self, obfuscation_algorithm_ref=None, refanging_transform_type=None, has_changed=None, delimiter='##comma##', pattern_type=None, datatype='string', refanging_transform=None, is_case_sensitive=True, bit_mask=None, appears_random=None, observed_encoding=None, defanging_algorithm_ref=None, is_obfuscated=None, regex_syntax=None, apply_condition='ANY', trend=None, idref=None, is_defanged=None, id=None, condition=None, valueOf_=None):
        super(KernelHookType, self).__init__(obfuscation_algorithm_ref, refanging_transform_type, has_changed, delimiter, pattern_type, datatype, refanging_transform, is_case_sensitive, bit_mask, appears_random, observed_encoding, defanging_algorithm_ref, is_obfuscated, regex_syntax, apply_condition, trend, idref, is_defanged, id, condition, valueOf_)
        self.datatype = _cast(None, datatype)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if KernelHookType.subclass:
            return KernelHookType.subclass(*args_, **kwargs_)
        else:
            return KernelHookType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_datatype(self): return self.datatype
    def set_datatype(self, datatype): self.datatype = datatype
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_ or
            super(KernelHookType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinKernelHookObj:', name_='KernelHookType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='KernelHookType')
        if self.hasContent_():
            lwrite('>')
            lwrite(quote_xml(self.valueOf_))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinKernelHookObj:', name_='KernelHookType'):
        super(KernelHookType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='KernelHookType')
        if self.datatype is not None:

            lwrite(' datatype=%s' % (quote_attrib(self.datatype), ))
    def exportChildren(self, lwrite, level, namespace_='WinKernelHookObj:', name_='KernelHookType', fromsubclass_=False, pretty_print=True):
        super(KernelHookType, self).exportChildren(lwrite, level, 'WinKernelHookObj:', name_, True, pretty_print=pretty_print)
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('datatype', node)
        if value is not None:

            self.datatype = value
        super(KernelHookType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class KernelHookType

class WindowsKernelHookObjectType(cybox_common.ObjectPropertiesType):
    """The WindowsKernelHookObjectType type is intended to characterize
    Windows kernel function hooks."""

    subclass = None
    superclass = cybox_common.ObjectPropertiesType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, Digital_Signature_Hooking=None, Digital_Signature_Hooked=None, Hooking_Address=None, Hook_Description=None, Hooked_Function=None, Hooked_Module=None, Hooking_Module=None, Type=None):
        super(WindowsKernelHookObjectType, self).__init__(object_reference, Custom_Properties, xsi_type )
        self.Digital_Signature_Hooking = Digital_Signature_Hooking
        self.Digital_Signature_Hooked = Digital_Signature_Hooked
        self.Hooking_Address = Hooking_Address
        self.Hook_Description = Hook_Description
        self.Hooked_Function = Hooked_Function
        self.Hooked_Module = Hooked_Module
        self.Hooking_Module = Hooking_Module
        self.Type = Type
    def factory(*args_, **kwargs_):
        if WindowsKernelHookObjectType.subclass:
            return WindowsKernelHookObjectType.subclass(*args_, **kwargs_)
        else:
            return WindowsKernelHookObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Digital_Signature_Hooking(self): return self.Digital_Signature_Hooking
    def set_Digital_Signature_Hooking(self, Digital_Signature_Hooking): self.Digital_Signature_Hooking = Digital_Signature_Hooking
    def get_Digital_Signature_Hooked(self): return self.Digital_Signature_Hooked
    def set_Digital_Signature_Hooked(self, Digital_Signature_Hooked): self.Digital_Signature_Hooked = Digital_Signature_Hooked
    def get_Hooking_Address(self): return self.Hooking_Address
    def set_Hooking_Address(self, Hooking_Address): self.Hooking_Address = Hooking_Address
    def validate_UnsignedLongObjectPropertyType(self, value):
        # Validate type cybox_common.UnsignedLongObjectPropertyType, a restriction on None.
        pass
    def get_Hook_Description(self): return self.Hook_Description
    def set_Hook_Description(self, Hook_Description): self.Hook_Description = Hook_Description
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_Hooked_Function(self): return self.Hooked_Function
    def set_Hooked_Function(self, Hooked_Function): self.Hooked_Function = Hooked_Function
    def get_Hooked_Module(self): return self.Hooked_Module
    def set_Hooked_Module(self, Hooked_Module): self.Hooked_Module = Hooked_Module
    def get_Hooking_Module(self): return self.Hooking_Module
    def set_Hooking_Module(self, Hooking_Module): self.Hooking_Module = Hooking_Module
    def get_Type(self): return self.Type
    def set_Type(self, Type): self.Type = Type
    def validate_KernelHookType(self, value):
        # Validate type KernelHookType, a restriction on None.
        pass
    def hasContent_(self):
        if (
            self.Digital_Signature_Hooking is not None or
            self.Digital_Signature_Hooked is not None or
            self.Hooking_Address is not None or
            self.Hook_Description is not None or
            self.Hooked_Function is not None or
            self.Hooked_Module is not None or
            self.Hooking_Module is not None or
            self.Type is not None or
            super(WindowsKernelHookObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinKernelHookObj:', name_='WindowsKernelHookObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsKernelHookObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinKernelHookObj:', name_='WindowsKernelHookObjectType'):
        super(WindowsKernelHookObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsKernelHookObjectType')
    def exportChildren(self, lwrite, level, namespace_='WinKernelHookObj:', name_='WindowsKernelHookObjectType', fromsubclass_=False, pretty_print=True):
        super(WindowsKernelHookObjectType, self).exportChildren(lwrite, level, 'WinKernelHookObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Digital_Signature_Hooking is not None:
            self.Digital_Signature_Hooking.export(lwrite, level, 'WinKernelHookObj:', name_='Digital_Signature_Hooking', pretty_print=pretty_print)
        if self.Digital_Signature_Hooked is not None:
            self.Digital_Signature_Hooked.export(lwrite, level, 'WinKernelHookObj:', name_='Digital_Signature_Hooked', pretty_print=pretty_print)
        if self.Hooking_Address is not None:
            self.Hooking_Address.export(lwrite, level, 'WinKernelHookObj:', name_='Hooking_Address', pretty_print=pretty_print)
        if self.Hook_Description is not None:
            self.Hook_Description.export(lwrite, level, 'WinKernelHookObj:', name_='Hook_Description', pretty_print=pretty_print)
        if self.Hooked_Function is not None:
            self.Hooked_Function.export(lwrite, level, 'WinKernelHookObj:', name_='Hooked_Function', pretty_print=pretty_print)
        if self.Hooked_Module is not None:
            self.Hooked_Module.export(lwrite, level, 'WinKernelHookObj:', name_='Hooked_Module', pretty_print=pretty_print)
        if self.Hooking_Module is not None:
            self.Hooking_Module.export(lwrite, level, 'WinKernelHookObj:', name_='Hooking_Module', pretty_print=pretty_print)
        if self.Type is not None:
            self.Type.export(lwrite, level, 'WinKernelHookObj:', name_='Type', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(WindowsKernelHookObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Digital_Signature_Hooking':
            obj_ = cybox_common.DigitalSignatureInfoType.factory()
            obj_.build(child_)
            self.set_Digital_Signature_Hooking(obj_)
        elif nodeName_ == 'Digital_Signature_Hooked':
            obj_ = cybox_common.DigitalSignatureInfoType.factory()
            obj_.build(child_)
            self.set_Digital_Signature_Hooked(obj_)
        elif nodeName_ == 'Hooking_Address':
            obj_ = cybox_common.UnsignedLongObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Hooking_Address(obj_)
        elif nodeName_ == 'Hook_Description':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Hook_Description(obj_)
        elif nodeName_ == 'Hooked_Function':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Hooked_Function(obj_)
        elif nodeName_ == 'Hooked_Module':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Hooked_Module(obj_)
        elif nodeName_ == 'Hooking_Module':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Hooking_Module(obj_)
        elif nodeName_ == 'Type':
            obj_ = KernelHookType.factory()
            obj_.build(child_)
            self.set_Type(obj_)
        super(WindowsKernelHookObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class WindowsKernelHookObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Errors': cybox_common.ErrorsType,
    'Time': cybox_common.TimeType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Digital_Signature_Hooking': cybox_common.DigitalSignatureInfoType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'SubDatum': cybox_common.MetadataType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Value': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'Hooked_Module': cybox_common.StringObjectPropertyType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Functions': cybox_common.FunctionsType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'System': cybox_common.ObjectPropertiesType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Type': cybox_common.ControlledVocabularyStringType,
    'Hooked_Function': cybox_common.StringObjectPropertyType,
    'Compilers': cybox_common.CompilersType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'String': cybox_common.ExtractedStringType,
    'Tool': cybox_common.ToolInformationType,
    'Build_Information': cybox_common.BuildInformationType,
    'Tool_Hashes': cybox_common.HashListType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Language': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'Reference_Description': cybox_common.StructuredTextType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Import': cybox_common.StringObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Dependencies': cybox_common.DependenciesType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateRangeType,
    'Hashes': cybox_common.HashListType,
    'Segments': cybox_common.HashSegmentsType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Error': cybox_common.ErrorType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'Contributors': cybox_common.PersonnelType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'Imports': cybox_common.ImportsType,
    'Library': cybox_common.LibraryType,
    'References': cybox_common.ToolReferencesType,
    'Hook_Description': cybox_common.StringObjectPropertyType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Libraries': cybox_common.LibrariesType,
    'Hooking_Module': cybox_common.StringObjectPropertyType,
    'Function': cybox_common.StringObjectPropertyType,
    'Description': cybox_common.StructuredTextType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'Hooking_Address': cybox_common.UnsignedLongObjectPropertyType,
    'Address': cybox_common.HexBinaryObjectPropertyType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'Compiler': cybox_common.CompilerType,
    'Name': cybox_common.StringObjectPropertyType,
    'Digital_Signature_Hooked': cybox_common.DigitalSignatureInfoType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Data_Size': cybox_common.DataSizeType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'Contributor': cybox_common.ContributorType,
    'Tools': cybox_common.ToolsInformationType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Kernel_Hook'
        rootClass = WindowsKernelHookObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_=rootTag,
#        namespacedef_='',
#        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Kernel_Hook'
        rootClass = WindowsKernelHookObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Kernel_Hook'
        rootClass = WindowsKernelHookObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_="Windows_Kernel_Hook",
#        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "WindowsKernelHookObjectType",
    "KernelHookType"
    ]



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.address_object import Address
from cybox.objects.network_socket_object import NetworkSocket, SocketOptions
from cybox.test import EntityTestCase
from cybox.test.objects import ObjectTestCase


class TestSocketOptions(EntityTestCase, unittest.TestCase):
    klass = SocketOptions

    _full_dict = {
        'ip_multicast_if': u("eth1"),  # Should probably be a boolean
        'ip_multicast_if2': u("wlan0"),  # Should probably be a boolean
        'ip_multicast_loop': False,
        'ip_tos': u("true"),  # Should probably be a boolean
        'so_broadcast': False,
        'so_conditional_accept': True,
        'so_keepalive': False,
        'so_dontroute': True,
        'so_linger': 17,  # Should probably be a boolean
        'so_dontlinger': True,
        'so_oobinline': False,
        'so_rcvbuf': 44,  # Should probably be a boolean
        'so_group_priority': 19,
        'so_reuseaddr': True,
        'so_debug': False,
        'so_rcvtimeo': 42,  # Should this allow both seconds and microseconds?
        'so_sndbuf': 1000,
        'so_sndtimeo': 22,  # Should this allow both seconds and microseconds?
        'so_update_accept_context': 3,  # Should probably be a boolean
        'so_timeout': 99,
        'tcp_nodelay': False,
    }


class TestNetworkSocket(ObjectTestCase, unittest.TestCase):
    object_type = "NetworkSocketObjectType"
    klass = NetworkSocket

    _full_dict = {
        'is_blocking': False,
        'is_listening': True,
        'address_family': u("AF_INET"),
        'domain': u("PF_INET"),
        'local_address': {
            'ip_address': {
                'address_value': u("192.168.1.4"),
                'category': Address.CAT_IPV4,
                'xsi:type': "AddressObjectType",
            },
            'xsi:type': "SocketAddressObjectType",
        },
        'options': {'so_broadcast': False},
        'protocol': u("IPPROTO_TCP"),
        'remote_address': {
            'ip_address': {
                'address_value': u("192.168.100.55"),
                'category': Address.CAT_IPV4,
                'xsi:type': "AddressObjectType",
            },
            'port': {
                'port_value': 80,
                'xsi:type': "PortObjectType",
            },
            'xsi:type': "SocketAddressObjectType",
        },
        'type': u("SOCK_STREAM"),
        'socket_descriptor': 567,
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.cybox_common as common_binding
from cybox.common import DateWithPrecision


class DateRange(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.DateRangeType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/common-2'

    start_date = fields.TypedField("Start_Date", DateWithPrecision)
    end_date = fields.TypedField("End_Date", DateWithPrecision)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.win_user_account_object as win_user_binding
from cybox.common import String
from cybox.objects.user_account_object import (GroupList, PrivilegeList,
        UserAccount)


class WinGroup(entities.Entity):
    _binding = win_user_binding
    _binding_class = win_user_binding.WindowsGroupType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-user-account-2'

    name = fields.TypedField("Name", String)

    # TODO: Allow these to be represented as single strings in to/from_dict


class WinGroupList(GroupList):
    group = fields.TypedField("Group", WinGroup, multiple=True)


class WinPrivilege(entities.Entity):
    _binding = win_user_binding
    _binding_class = win_user_binding.WindowsPrivilegeType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-user-account-2'

    user_right = fields.TypedField("User_Right", String)

    # TODO: Allow these to be represented as single strings in to/from_dict


class WinPrivilegeList(PrivilegeList):
    privilege = fields.TypedField("Privilege", WinPrivilege, multiple=True)


class WinUser(UserAccount):
    _binding = win_user_binding
    _binding_class = win_user_binding.WindowsUserAccountObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-user-account-2'
    _XSI_NS = "WinUserAccountObj"
    _XSI_TYPE = "WindowsUserAccountObjectType"

    security_id = fields.TypedField('Security_ID', String)
    security_type = fields.TypedField('Security_Type', String)

    #Override abstract types here
    group_list = fields.TypedField('Group_List', WinGroupList)
    privilege_list = fields.TypedField('Privilege_List', WinPrivilegeList)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common
from . import file_object


class ArchiveFileFormatType(cybox_common.BaseObjectPropertyType):
    """The ArchiveFileFormatType specifies archive file formats via a union
    of the ArchiveFileFormatEnum type and the atomic xs:string type.
    Its base type is the CybOX Core cybox_common.BaseObjectPropertyType, for
    permitting complex (i.e. regular-expression based)
    specifications."""

    subclass = None
    superclass = cybox_common.BaseObjectPropertyType
    def __init__(self, obfuscation_algorithm_ref=None, refanging_transform_type=None, has_changed=None, delimiter='##comma##', pattern_type=None, datatype='string', refanging_transform=None, is_case_sensitive=True, bit_mask=None, appears_random=None, observed_encoding=None, defanging_algorithm_ref=None, is_obfuscated=None, regex_syntax=None, apply_condition='ANY', trend=None, idref=None, is_defanged=None, id=None, condition=None, valueOf_=None):
        super(ArchiveFileFormatType, self).__init__(obfuscation_algorithm_ref, refanging_transform_type, has_changed, delimiter, pattern_type, datatype, refanging_transform, is_case_sensitive, bit_mask, appears_random, observed_encoding, defanging_algorithm_ref, is_obfuscated, regex_syntax, apply_condition, trend, idref, is_defanged, id, condition, valueOf_, )
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if ArchiveFileFormatType.subclass:
            return ArchiveFileFormatType.subclass(*args_, **kwargs_)
        else:
            return ArchiveFileFormatType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_ or
            super(ArchiveFileFormatType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='ArchiveFileObj:', name_='ArchiveFileFormatType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='ArchiveFileFormatType')
        if self.hasContent_():
            lwrite('>')
            lwrite(quote_xml(self.valueOf_))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='ArchiveFileObj:', name_='ArchiveFileFormatType'):
        super(ArchiveFileFormatType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='ArchiveFileFormatType')
    def exportChildren(self, lwrite, level, namespace_='ArchiveFileObj:', name_='ArchiveFileFormatType', fromsubclass_=False, pretty_print=True):
        super(ArchiveFileFormatType, self).exportChildren(lwrite, level, 'ArchiveFileObj:', name_, True, pretty_print=pretty_print)
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(ArchiveFileFormatType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class ArchiveFileFormatType

class ArchiveFileObjectType(file_object.FileObjectType):
    """The ArchiveFileObjectType type is intended to characterize archive
    files."""

    subclass = None
    superclass = file_object.FileObjectType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, is_packed=None, is_masqueraded=None, File_Name=None, File_Path=None, Device_Path=None, Full_Path=None, File_Extension=None, Size_In_Bytes=None, Magic_Number=None, File_Format=None, Hashes=None, Digital_Signatures=None, Modified_Time=None, Accessed_Time=None, Created_Time=None, File_Attributes_List=None, Permissions=None, User_Owner=None, Packer_List=None, Peak_Entropy=None, Sym_Links=None, Byte_Runs=None, Extracted_Features=None, Encryption_Algorithm=None, Decryption_Key=None, Compression_Method=None, Compression_Version=None, Compression_Comment=None, Archive_Format=None, Version=None, File_Count=None, Comment=None, Archived_File=None):
        super(ArchiveFileObjectType, self).__init__(object_reference, Custom_Properties, xsi_type, is_packed, is_masqueraded, File_Name, File_Path, Device_Path, Full_Path, File_Extension, Size_In_Bytes, Magic_Number, File_Format, Hashes, Digital_Signatures, Modified_Time, Accessed_Time, Created_Time, File_Attributes_List, Permissions, User_Owner, Packer_List, Peak_Entropy, Sym_Links, Byte_Runs, Extracted_Features, Encryption_Algorithm, Decryption_Key, Compression_Method, Compression_Version, Compression_Comment, )
        self.Archive_Format = Archive_Format
        self.Version = Version
        self.File_Count = File_Count
        self.Encryption_Algorithm = Encryption_Algorithm
        self.Decryption_Key = Decryption_Key
        self.Comment = Comment
        if Archived_File is None:
            self.Archived_File = []
        else:
            self.Archived_File = Archived_File
    def factory(*args_, **kwargs_):
        if ArchiveFileObjectType.subclass:
            return ArchiveFileObjectType.subclass(*args_, **kwargs_)
        else:
            return ArchiveFileObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Archive_Format(self): return self.Archive_Format
    def set_Archive_Format(self, Archive_Format): self.Archive_Format = Archive_Format
    def validate_ArchiveFileFormatType(self, value):
        # Validate type ArchiveFileFormatType, a restriction on None.
        pass
    def get_Version(self): return self.Version
    def set_Version(self, Version): self.Version = Version
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_File_Count(self): return self.File_Count
    def set_File_Count(self, File_Count): self.File_Count = File_Count
    def validate_IntegerObjectPropertyType(self, value):
        # Validate type cybox_common.IntegerObjectPropertyType, a restriction on None.
        pass
    def get_Encryption_Algorithm(self): return self.Encryption_Algorithm
    def set_Encryption_Algorithm(self, Encryption_Algorithm): self.Encryption_Algorithm = Encryption_Algorithm
    def validate_CipherType(self, value):
        # Validate type cybox_common.CipherType, a restriction on None.
        pass
    def get_Decryption_Key(self): return self.Decryption_Key
    def set_Decryption_Key(self, Decryption_Key): self.Decryption_Key = Decryption_Key
    def get_Comment(self): return self.Comment
    def set_Comment(self, Comment): self.Comment = Comment
    def get_Archived_File(self): return self.Archived_File
    def set_Archived_File(self, Archived_File): self.Archived_File = Archived_File
    def add_Archived_File(self, value): self.Archived_File.append(value)
    def insert_Archived_File(self, index, value): self.Archived_File[index] = value
    def hasContent_(self):
        if (
            self.Archive_Format is not None or
            self.Version is not None or
            self.File_Count is not None or
            self.Encryption_Algorithm is not None or
            self.Decryption_Key is not None or
            self.Comment is not None or
            self.Archived_File or
            super(ArchiveFileObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='ArchiveFileObj:', name_='ArchiveFileObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='ArchiveFileObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='ArchiveFileObj:', name_='ArchiveFileObjectType'):
        super(ArchiveFileObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='ArchiveFileObjectType')
    def exportChildren(self, lwrite, level, namespace_='ArchiveFileObj:', name_='ArchiveFileObjectType', fromsubclass_=False, pretty_print=True):
        super(ArchiveFileObjectType, self).exportChildren(lwrite, level, 'ArchiveFileObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Archive_Format is not None:
            self.Archive_Format.export(lwrite, level, 'ArchiveFileObj:', name_='Archive_Format', pretty_print=pretty_print)
        if self.Version is not None:
            self.Version.export(lwrite, level, 'ArchiveFileObj:', name_='Version', pretty_print=pretty_print)
        if self.File_Count is not None:
            self.File_Count.export(lwrite, level, 'ArchiveFileObj:', name_='File_Count', pretty_print=pretty_print)
        if self.Encryption_Algorithm is not None:
            self.Encryption_Algorithm.export(lwrite, level, 'ArchiveFileObj:', name_='Encryption_Algorithm', pretty_print=pretty_print)
        if self.Decryption_Key is not None:
            self.Decryption_Key.export(lwrite, level, 'ArchiveFileObj:', name_='Decryption_Key', pretty_print=pretty_print)
        if self.Comment is not None:
            self.Comment.export(lwrite, level, 'ArchiveFileObj:', name_='Comment', pretty_print=pretty_print)
        for Archived_File_ in self.Archived_File:
            Archived_File_.export(lwrite, level, 'ArchiveFileObj:', name_='Archived_File', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(ArchiveFileObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Archive_Format':
            obj_ = ArchiveFileFormatType.factory()
            obj_.build(child_)
            self.set_Archive_Format(obj_)
        elif nodeName_ == 'Version':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Version(obj_)
        elif nodeName_ == 'File_Count':
            obj_ = cybox_common.IntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_File_Count(obj_)
        elif nodeName_ == 'Encryption_Algorithm':
            obj_ = cybox_common.CipherType.factory()
            obj_.build(child_)
            self.set_Encryption_Algorithm(obj_)
        elif nodeName_ == 'Decryption_Key':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Decryption_Key(obj_)
        elif nodeName_ == 'Comment':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Comment(obj_)
        elif nodeName_ == 'Archived_File':
            obj_ = file_object.FileObjectType.factory()
            obj_.build(child_)
            self.Archived_File.append(obj_)
        super(ArchiveFileObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class ArchiveFileObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Byte_Order': cybox_common.EndiannessType,
    'Errors': cybox_common.ErrorsType,
    'File_Extension': cybox_common.StringObjectPropertyType,
    'Time': cybox_common.TimeType,
    'Sym_Links': file_object.SymLinksListType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Size_In_Bytes': cybox_common.UnsignedLongObjectPropertyType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'Compression_Version': cybox_common.StringObjectPropertyType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'Byte_Runs': cybox_common.ByteRunsType,
    'SubDatum': cybox_common.MetadataType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Value': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'Device_Path': cybox_common.StringObjectPropertyType,
    'Produced_Time': cybox_common.DateTimeWithPrecisionType,
    'Reference': cybox_common.ToolReferenceType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Start_Date': cybox_common.DateWithPrecisionType,
    'Functions': cybox_common.FunctionsType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'Start_Time': cybox_common.DateTimeWithPrecisionType,
    'System': cybox_common.ObjectPropertiesType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Version': cybox_common.StringObjectPropertyType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Created_Time': cybox_common.DateTimeObjectPropertyType,
    'Type': file_object.PackerClassType,
    'Accessed_Time': cybox_common.DateTimeObjectPropertyType,
    'Compilers': cybox_common.CompilersType,
    'Digital_Signatures': cybox_common.DigitalSignaturesType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'String': cybox_common.ExtractedStringType,
    'File_Format': cybox_common.StringObjectPropertyType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
    'Build_Information': cybox_common.BuildInformationType,
    'Detected_Entrypoint_Signatures': file_object.EntryPointSignatureListType,
    'Tool_Hashes': cybox_common.HashListType,
    'File_Path': file_object.FilePathType,
    'Observable_Location': cybox_common.LocationType,
    'Entry_Point_Signature': file_object.EntryPointSignatureType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'End_Date': cybox_common.DateWithPrecisionType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Compensation_Model': cybox_common.CompensationModelType,
    'Signature': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'Comment': cybox_common.StringObjectPropertyType,
    'User_Owner': cybox_common.StringObjectPropertyType,
    'Contributors': cybox_common.PersonnelType,
    'Packer': file_object.PackerType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'File_Attributes_List': file_object.FileAttributeType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Received_Time': cybox_common.DateTimeWithPrecisionType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Packer_List': file_object.PackerListType,
    'Import': cybox_common.StringObjectPropertyType,
    'Sym_Link': cybox_common.StringObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Dependencies': cybox_common.DependenciesType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateRangeType,
    'Hashes': cybox_common.HashListType,
    'Segments': cybox_common.HashSegmentsType,
    'Permissions': file_object.FilePermissionsType,
    'Language': cybox_common.StringObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Error': cybox_common.ErrorType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'Imports': cybox_common.ImportsType,
    'Opcodes': cybox_common.StringObjectPropertyType,
    'Library': cybox_common.LibraryType,
    'Archived_File': file_object.FileObjectType,
    'References': cybox_common.ToolReferencesType,
    'Encryption_Algorithm': cybox_common.CipherType,
    'Compilation_Date': cybox_common.DateTimeWithPrecisionType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'EP_Jump_Codes': file_object.EPJumpCodeType,
    'File_Name': cybox_common.StringObjectPropertyType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Observation_Location': cybox_common.LocationType,
    'Modified_Time': cybox_common.DateTimeObjectPropertyType,
    'Libraries': cybox_common.LibrariesType,
    'Function': cybox_common.StringObjectPropertyType,
    'Description': cybox_common.StructuredTextType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'Extracted_Features': cybox_common.ExtractedFeaturesType,
    'Magic_Number': cybox_common.HexBinaryObjectPropertyType,
    'Decryption_Key': cybox_common.StringObjectPropertyType,
    'Address': cybox_common.HexBinaryObjectPropertyType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'Full_Path': cybox_common.StringObjectPropertyType,
    'Compiler': cybox_common.CompilerType,
    'Name': cybox_common.StringObjectPropertyType,
    'Depth': cybox_common.IntegerObjectPropertyType,
    'Entry_Point': cybox_common.HexBinaryObjectPropertyType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compression_Comment': cybox_common.StringObjectPropertyType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Data_Size': cybox_common.DataSizeType,
    'Compression_Method': cybox_common.StringObjectPropertyType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'File_Count': cybox_common.IntegerObjectPropertyType,
    'End_Time': cybox_common.DateTimeWithPrecisionType,
    'File': file_object.FileObjectType,
    'Contributor': cybox_common.ContributorType,
    'Peak_Entropy': cybox_common.DoubleObjectPropertyType,
    'Tools': cybox_common.ToolsInformationType,
    'Tool': cybox_common.ToolInformationType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Archive_File'
        rootClass = ArchiveFileObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    sys.stdout.write('<?xml version="1.0" ?>\n')
    rootObj.export(sys.stdout.write, 0, name_=rootTag,
        namespacedef_='',
        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Archive_File'
        rootClass = ArchiveFileObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Archive_File'
        rootClass = ArchiveFileObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    sys.stdout.write('<?xml version="1.0" ?>\n')
    rootObj.export(sys.stdout.write, 0, name_="Archive_File",
        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "ArchiveFileObjectType",
    "ArchiveFileFormatType"
    ]



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.compat import long
from cybox.objects.win_registry_key_object import WinRegistryKey
from cybox.test.objects import ObjectTestCase


class TestWinRegistryKey(ObjectTestCase, unittest.TestCase):
    object_type = "WindowsRegistryKeyObjectType"
    klass = WinRegistryKey

    _full_dict = {
        'key': u("\\SOFTWARE\\Microsoft\\Windows\\Windows Error Reporting"),
        'hive': u("HKEY_LOCAL_MACHINE"),
        'number_values': 6,
        'values': [
            {
                'name': u("Disabled"),
                'data': u("1"),
                'datatype': u("REG_DWORD"),
                'byte_runs': [{'length': 1, 'byte_run_data': u("A")}],
            },
            {
                'name': u("ErrorPort"),
                'data': u("\\WindowsErrorReportingServicePort"),
                'datatype': u("REG_SZ"),
            },
        ],
        'modified_time': u("2013-08-08T15:15:15-04:00"),
        'creator_username': u("gback"),
        'handle_list': [
            {
                'name': u("RegHandle"),
                'pointer_count': long(1),
                'type': u("RegistryKey"),
                'xsi:type': u('WindowsHandleObjectType'),
            },
        ],
        'number_subkeys': 1,
        'subkeys': [
            {
                'key': u("Consent"),
                'number_values': 1,
                'values': [
                    {
                        'name': u("NewUserDefaultConsent"),
                        'data': u("1"),
                        'datatype': u("REG_DWORD"),
                    },
                ],
                'xsi:type': 'WindowsRegistryKeyObjectType',
            },
        ],
        'byte_runs': [
            {'length': 4, 'byte_run_data': u("z!%f")},
            {'offset': 0x1000, 'length': 8, 'byte_run_data': u("%40V.,2@")},
        ],
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.common import MeasureSource
from cybox.test import EntityTestCase


class TestMeasureSource(EntityTestCase, unittest.TestCase):
    klass = MeasureSource

    _full_dict = {
        'class': "Software",
        'source_type': "Information Source",
        'name': "ASource",
        'information_source_type': u("Web Logs"),
        'tool_type': u("Vulnerability Scanner"),
        'description': u("A description of the source"),
        'contributors': [
            {
                'name': u("An amazing dude"),
                'email': u("amazing@dude.com"),
            },
            {
                'name': u("Another amazing dude"),
                'role': u("President of Amazing"),
                'organization': u("AmazingCo."),
            },
        ],
        'time': {
            'start_time': "2013-02-14T11:28:42-05:00",
            'end_time': "2014-03-11T06:22:17-05:00",
        },
        'tools': [
            {'name': u("AmazingTool (TM)")}
        ],
        'platform': {'description': u("The best platform")},
        #TODO: Add System and Instance
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from cybox.common import DigitalSignature, DigitalSignatureList
import cybox.test
from cybox.test import EntityTestCase


class TestDigitalSignature(EntityTestCase, unittest.TestCase):
    klass = DigitalSignature

    _full_dict = {
        'signature_exists': True,
        'signature_verified': False,
        'certificate_issuer': "SomeIssuer",
        'certificate_subject': "The Subject",
        'signature_description': "A Fake Signature",
    }


class TestDigitalSignatureList(unittest.TestCase):

    def test_round_trip(self):
        siglist_list = [
                {'certificate_issuer': "IssuerA", 'signature_verified': True},
                {'signature_description': "A Signature"},
            ]
        siglist_list2 = cybox.test.round_trip_list(DigitalSignatureList,
                                                       siglist_list)
        self.assertEqual(siglist_list, siglist_list2)


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.win_network_route_entry_object as win_network_route_entry_binding
from cybox.objects.network_route_entry_object import NetworkRouteEntry
from cybox.common import String


class WinNetworkRouteEntry(NetworkRouteEntry):
    _binding = win_network_route_entry_binding
    _binding_class = win_network_route_entry_binding.WindowsNetworkRouteEntryObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-network-route-entry-2"
    _XSI_NS = "WinNetworkRouteEntryObj"
    _XSI_TYPE = "WindowsNetworkRouteEntryObjectType"

    nl_route_protocol = fields.TypedField("NL_ROUTE_PROTOCOL", String)
    nl_route_origin = fields.TypedField("NL_ROUTE_ORIGIN", String)
